#include <msp.h>
#include "common.h"
#include "adc.h"
#include "fifo_function_queue.h"

static unsigned int  adcval;

//Callback function for queue
void (*temperature_ready_callback)();

void initializeADC(){

    //REF_A, waits until reference generator is not busy
    while(REFCTL0 &  REF_A_CTL0_GENBUSY);

    //Set REF_A to be used, temp sensor enabled and REFOUT disabled
    REFCTL0 = REF_A_CTL0_ON;

    //Sets ADC14ENC to 0 to make modifications to the registers, resets all register just in case
    ADC14->CTL0 = 0;

    //ADC Clock Divided by 3, ACLK, Single Channel - Single Conversion, SH signal sourced from sampling timer
    ADC14->CTL0 |= ADC14_CTL0_DIV_2 + ADC14_CTL0_SSEL_2 + ADC14_CTL0_SHP;

    //TODO Make sure time makes sense when timer is configured - Sample rate must not exceed 200ksps
    //Sets default value to CTL1 register and
    //sets resolution to 10 bits, low power mode (sample rate must not exceed 200ksps), temperature sensor selected for ADC input channel MAX - 1, ADC14CSTARTADDx = 0
    //ADC14->CTL1 = ADC14_CTL1_RES_1 + ADC14_CTL1_PWRMD_2 + ADC14_CTL1_TCMAP;

    ADC14->CTL1 = ADC14_CTL1_RES_1 + ADC14_CTL1_TCMAP;

    //Sets Channel 22 as input channel, single ended mode, V(R+) = Vref buffered / V(R-) = Vss
    ADC14->MCTL[0]= ADC14_MCTLN_INCH_22 + ADC14_MCTLN_VRSEL_1;

    //TODO Review this function
    //Wait for ADC to stabilize
    __delay_cycles(1000);
}

void setTempCallbackFunction(void *callback_function)
{
    temperature_ready_callback = callback_function;
}

void runADCConversion(){

    //ADC14 is turned on, enable conversion, start conversion,

    ADC14->CTL0 |= ADC14_CTL0_ON + ADC14_CTL0_ENC + ADC14_CTL0_SC;

    //Enable ISRs (ADC Register 0 - Channel 22)
    ADC14 -> IER0 |= ADC14_IER0_IE0;
    NVIC_SetPriority(ADC14_IRQn,3);
    NVIC_EnableIRQ(ADC14_IRQn);
}

int getTemperatureReading(){
    int  temp;
    float aux;
    //Converts ADC Reading to mVolts (1.2 Volts set for reference voltage)
    aux = (float)(adcval) * REF_VOLTAGE_DIV_BY_ADC_10BITS_RESOLUTION;

    //Converts mVolts to celsius using temperature sensor transfer function (no calibration)
    //Using 10 bits resolution
    temp = (int)((aux - 686) * 0.512);

    return temp;
}

void ADC14_IRQHandler(void)
{
    //Reading to ADC14MEM clears the IFG
    adcval = ADC14->MEM[0];
    ADC14->CTL0 &= ~ADC14_CTL0_ENC;
    ADC14->CTL0 &= ~ADC14_CTL0_ON;

    //Disable ISRs
    ADC14 -> IER0 &= ~ADC14_IER0_IE0;
    NVIC_DisableIRQ(ADC14_IRQn);

    addToFunctionQueue(temperature_ready_callback);
    __low_power_mode_off_on_exit();
}
